import {
  Box,
  MenuItem,
  Select,
  SelectChangeEvent,
  Stack,
  TextField,
  Typography,
} from '@mui/material'
import { SubmitHandler, useForm } from 'react-hook-form'
import React, { forwardRef, Ref, useImperativeHandle, useState } from 'react'
import { useTranslation } from 'react-i18next'
import {
  CountryCallingCode,
  getCountries,
  getCountryCallingCode,
} from 'libphonenumber-js'
import { ContactInformationBySponsor } from './guestDataForm'
import { getName } from 'i18n-iso-countries'
import { EnteredGuestData } from './enteredGuestData'
import { GuestRegisterCallableMethods } from './guestRegisterCallableMethods'
import { isValidFnr, isValidMobilePhoneNumber } from '../../../utils'
import AuthenticationMethod from './authenticationMethod'
import { CountryCallingCodes } from 'libphonenumber-js/types'

interface GuestRegisterProperties {
  nextHandler(guestData: EnteredGuestData): void

  guestData: ContactInformationBySponsor
}

/**
 * This component is the form where the guest enters missing information about himself and
 * where he can see the data the sponsor has entered and the role. The page may also
 * be populated with data from a third-party like Feide if the guest logged in using that.
 */
const GuestRegisterStep = forwardRef(
  (props: GuestRegisterProperties, ref: Ref<GuestRegisterCallableMethods>) => {
    const { i18n, t } = useTranslation(['common'])
    const { nextHandler, guestData } = props

    const [countryCode, setCountryCode] = useState<
      CountryCallingCode | undefined
    >(undefined)

    const handleCountryCodeChange = (event: SelectChangeEvent) => {
      if (event.target.value) {
        const countryCodeType = getCountries().find((value) => {
          return value.toString() === event.target.value
        })
        if (countryCodeType) {
          setCountryCode(getCountryCallingCode(countryCodeType))
        }
      } else {
        setCountryCode(undefined)
      }
    }

    const submit: SubmitHandler<EnteredGuestData> = (data) => {
      nextHandler(data)
    }

    const {
      register,
      handleSubmit,
      setValue,
      trigger,
      formState: { errors },
    } = useForm()
    const onSubmit = handleSubmit<EnteredGuestData>(submit)

    function doSubmit() {
      return onSubmit()
    }

    register('mobilePhone', {
      required: t<string>('validation.roleTypeRequired'),
      validate: isValidMobilePhoneNumber,
    })

    useImperativeHandle(ref, () => ({ doSubmit }))

    return (
      <>
        <Typography
          variant="h5"
          sx={{
            paddingTop: '1rem',
            paddingBottom: '1rem',
          }}
        >
          {t('guestRegisterWizardText.yourContactInformation')}
        </Typography>
        <Typography sx={{ paddingBottom: '2rem' }}>
          {t('guestRegisterWizardText.contactInformationDescription')}
        </Typography>
        <Box sx={{ maxWidth: '30rem' }}>
          <form onSubmit={onSubmit}>
            <Stack spacing={2}>
              <TextField
                id="firstName"
                label={t('input.firstName')}
                value={guestData.first_name}
                disabled
              />
              <TextField
                id="lastName"
                label={t('input.lastName')}
                value={guestData.last_name}
                disabled
              />

              <TextField
                id="email"
                label={t('input.email')}
                value={guestData.email}
                disabled
              />

              <Box
                sx={{
                  display: 'flex',
                  flexDirection: 'row',
                  paddingBottom: '2rem',
                }}
              >
                <Select
                  sx={{
                    maxHeight: '2.5rem',
                    minWidth: '5rem',
                    marginRight: '0.5rem',
                  }}
                  labelId="phone-number-select"
                  id="phone-number-select"
                  onChange={handleCountryCodeChange}
                >
                  {getCountries().map((country) => (
                    <MenuItem key={country} value={country}>
                      {getName(country, 'en')} +{getCountryCallingCode(country)}
                    </MenuItem>
                  ))}
                </Select>

                <TextField
                  sx={{ flexGrow: 2 }}
                  label={t('input.mobilePhone')}
                  error={!!errors.mobilePhone}
                  helperText={errors.mobilePhone && errors.mobilePhone.message}
                  onChange={(value) => {
                    if (countryCode) {
                      // The country code and the rest of the mobile number are in two fields, so cannot
                      // register the field directly in form, but need to have extra logic defined
                      // to combine the values before writing them to the form handling
                      setValue(
                        'mobilePhone',
                        `+${countryCode.toString()}${value.target.value}`
                      )
                      trigger('mobilePhone')
                    }
                  }}
                />
              </Box>

              {guestData.user_information_source ===
                AuthenticationMethod.NationalIdNumberOrPassport && (
                <>
                  <TextField
                    id="passport"
                    data-testid="passport_number_input"
                    label={t('input.passportNumber')}
                    {...register('passportNumber', {
                      required: t<string>('validation.passportNumberRequired'),
                    })}
                  />

                  <TextField
                    id="national_id_number"
                    label={t('input.nationalIdNumber')}
                    error={!!errors.national_id_number}
                    helperText={
                      errors.nationalIdNumber && errors.nationalIdNumber.message
                    }
                    {...register('nationalIdNumber', {
                      validate: isValidFnr,
                    })}
                  />
                </>
              )}

              {guestData.user_information_source ===
                AuthenticationMethod.Feide && (
                <TextField
                  id="national_id_number"
                  data-testid="national_id_number_feide"
                  label={t('input.nationalIdNumber')}
                  disabled
                />
              )}

              <Typography variant="h5" sx={{ paddingTop: '1rem' }}>
                {t('guestRegisterWizardText.yourGuestPeriod')}
              </Typography>
              <Typography sx={{ paddingBottom: '1rem' }}>
                {t('guestRegisterWizardText.guestPeriodDescription')}
              </Typography>
              <TextField
                id="ou-unit"
                value={
                  i18n.language === 'en'
                    ? guestData.ou_name_en
                    : guestData.ou_name_nb
                }
                label={t('ou')}
                disabled
              />

              <TextField
                id="roleType"
                label={t('input.roleType')}
                value={
                  i18n.language === 'en'
                    ? guestData.role_name_en
                    : guestData.role_name_nb
                }
                disabled
              />

              <TextField
                id="rolePeriod"
                label={t('period')}
                value={`${guestData.role_start} - ${guestData.role_end}`}
                disabled
              />

              <TextField
                id="comment"
                label={t('input.comment')}
                multiline
                rows={5}
                value={guestData.comment}
                disabled
              />
            </Stack>
          </form>
        </Box>
      </>
    )
  }
)

export default GuestRegisterStep
